<?php

/*
 * Plugin Name: Expay Payment Gateway
 * Plugin URI:
 * Description: Allows you to use Expay gateway
 * Version: 2.1.9
 * Author: Expay Software Co., Ltd
*/

defined('ABSPATH') or die('No direct script access allowed.');

add_filter('woocommerce_payment_gateways', 'add_payment_gateway_class');
function add_payment_gateway_class($gateways)
{
    $gateways[] = 'PaymentGateway';
    return $gateways;
}

add_action('plugins_loaded', 'register_payment_gateway_class');
function register_payment_gateway_class()
{
    class PaymentGateway extends WC_Payment_Gateway
    {
        private $publishable_key;
        private $private_key;
        private $callback_secret;
        private $testmode;
        private $sandbox_url;
        private $live_url;

        public function __construct()
        {
            $this->include_classes();
            $this->define_constants();
            $this->init_plugin_settings();
            $this->session_init();
        }

        private function session_init()
        {
            WC()->session = new WC_Session_Handler();
            WC()->session->init();
            WC()->session->set('publishable_key', $this->publishable_key);
            WC()->session->set('private_key', $this->private_key);
            WC()->session->set('callback_secret', $this->callback_secret ?? $this->private_key);
            WC()->session->set('testmode', $this->testmode);
            WC()->session->set('live_url', $this->live_url);
            WC()->session->set('sandbox_url', $this->sandbox_url);
        }


        private function init_plugin_settings()
        {
            $this->id = PLUGIN_NAME;
            $this->has_fields = true;
            $this->order_button_text = __('Pay');
            $this->method_title = __(PLUGIN_TITLE . ' Gateway Merchant API', PLUGIN_NAME);

            $this->init_form_fields();

            $this->init_settings();
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->enabled = $this->get_option('enabled');
            $this->testmode = 'yes' === $this->get_option('testmode');
            $this->live_url = $this->get_option('live_url');
            $this->sandbox_url = $this->get_option('sandbox_url');
            $this->private_key = $this->testmode ? $this->get_option('test_private_key') : $this->get_option('private_key');
            $this->callback_secret = $this->testmode ? $this->get_option('test_callback_secret') : $this->get_option('callback_secret');
            $this->publishable_key = $this->testmode ? $this->get_option('test_publishable_key') : $this->get_option('publishable_key');
            $this->instructions = $this->get_option('instructions');

            global $woocommerce;
            if (version_compare($woocommerce->version, '2.0.0', '>=')) {
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
            } else {
                add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
            }
            add_action('admin_notices', array($this, 'admin_notices'));

            add_action('woocommerce_receipt_' . $this->id, array($this, 'waiting_form'));
            add_action('woocommerce_api_' . PLUGIN_NAME, array($this, 'webhook'), 10);
        }

        private function include_classes()
        {
            defined('WOOCOMMERCE_PLUGIN_PATH') || define('WOOCOMMERCE_PLUGIN_PATH', __DIR__);
            require_once WOOCOMMERCE_PLUGIN_PATH . '/includes/utils/PaymentProvider.php';
            require_once WOOCOMMERCE_PLUGIN_PATH . '/includes/utils/Curl.php';
            require_once WOOCOMMERCE_PLUGIN_PATH . '/includes/utils/PaymentHelper.php';
            require_once WOOCOMMERCE_PLUGIN_PATH . '/includes/events/EventHandler.php';
            require_once WOOCOMMERCE_PLUGIN_PATH . '/includes/events/BaseEvent.php';
        }

        private function define_constants()
        {
            $config = parse_ini_file(WOOCOMMERCE_PLUGIN_PATH . '/config/config.ini');
            defined('PLUGIN_NAME') || define('PLUGIN_NAME', $config['name']);
            defined('PLUGIN_TITLE') || define('PLUGIN_TITLE', $config['title']);
            defined('PLUGIN_SANDBOX_URL') || define('PLUGIN_SANDBOX_URL', $config['sandbox_url']);
            defined('PLUGIN_LIVE_URL') || define('PLUGIN_LIVE_URL', $config['live_url']);
            defined('PLUGIN_API_URL') || define('PLUGIN_API_URL', $config['api_url']);
        }

        public function admin_notices()
        {
            if ($this->enabled == 'no') {
                return false;
            }

            if (!$this->private_key && !$this->publishable_key) {
                echo '<div class="error"><p>' . __('You need to set a Publishable Key and Secret Key in settings to work. Please contact support <a href="' . PLUGIN_API_URL . '" target="_blank">' . PLUGIN_TITLE . '</a>.', 'beanstream-for-woocommerce') . '</p></div>';
                return false;
            }

            if (!$this->live_url && !$this->sandbox_url) {
                echo '<div class="error"><p>' . __('You need to set a Sandbox Url and Live Url in settings to work. Please contact support <a href="' . PLUGIN_API_URL . '" target="_blank">' . PLUGIN_TITLE . '</a>.', 'beanstream-for-woocommerce') . '</p></div>';
                return false;
            }
            return true;
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                'callback_url' => array(
                    'title' => 'Webhook url',
                    'type' => 'title',
                    'description' => 'Setup this URL as Callback URL: ' . add_query_arg('wc-api', PLUGIN_NAME, home_url('/')),
                ),
                'enabled' => array(
                    'title' => 'Enable/Disable',
                    'label' => 'Enable ' . PLUGIN_TITLE . ' Gateway',
                    'type' => 'checkbox',
                    'description' => '',
                    'default' => 'no'
                ),
                'title' => array(
                    'title' => 'Title',
                    'type' => 'text',
                    'default' => PLUGIN_TITLE,
                    'desc_tip' => true,
                ),
                'testmode' => array(
                    'title' => 'Sandbox mode',
                    'label' => 'Enable Sandbox Mode',
                    'type' => 'checkbox',
                    'description' => 'Place the payment gateway in sandbox mode using sandbox URL and API keys.',
                    'default' => 'yes',
                    'desc_tip' => true,
                ),
                'live_url' => array(
                    'title' => 'Live Url',
                    'default' => PLUGIN_LIVE_URL,
                    'type' => 'text'
                ),
                'sandbox_url' => array(
                    'title' => 'Sandbox Url',
                    'default' => PLUGIN_SANDBOX_URL,
                    'type' => 'text'
                ),
                'publishable_key' => array(
                    'title' => 'Live Merchant Key',
                    'type' => 'text'
                ),
                'private_key' => array(
                    'title' => 'Live Secret Key',
                    'type' => 'password'
                ),
                'callback_secret' => array(
                    'title' => 'Live Callback Signing Secret Key',
                    'type' => 'password'
                ),
                'test_publishable_key' => array(
                    'title' => 'Sandbox Merchant Key',
                    'type' => 'text'
                ),
                'test_private_key' => array(
                    'title' => 'Sandbox Secret Key',
                    'type' => 'password',
                ),
                'test_callback_secret' => array(
                    'title' => 'Sandbox Callback Signing Secret Key',
                    'type' => 'password',
                ),
            );
        }

        public function payment_fields()
        {
            $provider = new PaymentProvider();
            $payment_categories = $provider->getServicesCategories();
            $payment_methods = $provider->getMethodList();
            echo '<script type="text/javascript" src="https://code.jquery.com/jquery-latest.min.js"></script>
                  <script src="' . plugin_dir_url(__FILE__) . 'assets/js/jquery_form.js"></script>
                  <script src="https://unpkg.com/sweetalert/dist/sweetalert.min.js"></script>
                  <link rel="stylesheet" type="text/css" href=" ' . plugin_dir_url(__FILE__) . 'assets/css/main.css">
                  <script type="text/javascript" src="' . plugin_dir_url(__FILE__) . 'assets/js/index.js"></script>';

            if (is_array($payment_categories)) {
                foreach ($payment_categories as $category) {
                    if (count($category['services']) > 1) {
                        echo '<ul><input type="radio" class="wp-plugin_payment-category-radio" name="payment-category" id="' . $category['id'] . '" value="' . $category['key'] . '"/><label class="wp-plugin_payment-category-title" for="' . $category['id'] . '">' . $category['name'] . ' </label> <fieldset id="' . $category['key'] . '" class="wp-plugin_payment-field">';
                    } else {
                        echo '<ul>';
                    }
                    foreach ($category['services'] as $service_id) {
                        if (isset($payment_methods[$service_id])) {
                            if (count($category['services']) > 1) {
                                echo '<ul class="wp-plugin_banks-list">';
                                echo '<div class="item"><input class="wp-plugin_payment-method-radio" id="' . $payment_methods[$service_id]['id'] . '" type="radio" name="payment-method" value="' . $payment_methods[$service_id]['alias'] . '"/>';
                            } else {
                                echo '<div class="item"><input class="wp-plugin_payment-method-radio_2" id="' . $payment_methods[$service_id]['id'] . '" type="radio" name="payment-method" value="' . $payment_methods[$service_id]['alias'] . '"/>';
                            }
                            echo '<div class="wp-plugin_bank-label"><label for="' . $payment_methods[$service_id]['id'] . '"> ' . $payment_methods[$service_id]['name'] . '</label></div>';
                            if (isset($payment_methods[$service_id]['image'])) {
                                echo '<label for="' . $payment_methods[$service_id]['id'] . '">
                                  <div class="wp-plugin_bank-logo"><img src="' . $payment_methods[$service_id]['image'] . '"></div>
                                  </label>';
                            }
                            $payment_methods[$service_id]['attributes'] = array_filter($payment_methods[$service_id]['attributes'], function ($methods) {
                                return $methods['required'] == true;
                            });

                            echo '<div class="wp-plugin_attribute-list">';
                            foreach ($payment_methods[$service_id]['attributes'] as $key => $attribute) {
                                if ($key === 0) echo '<span>' . 'Please fill in the following fields:' . '</span>';
                                echo '<input class="wp-plugin_attribute" type="text" name="' . $attribute['key'] . '"data-regexp="' . htmlspecialchars($attribute['regexp']) . '"placeholder="' . $attribute['name'] . '" required disabled/>';
                            }
                            if (count($category['services']) > 1) {
                                echo '</div></div></ul>';
                            } else {
                                echo '</div></div>';
                            }
                        }
                    }
                    echo '</fieldset></ul>';
                }
            }
        }

        public function process_payment_error($message, $order)
        {
            $order->update_status('cancelled', $message);
            $order->add_order_note('Cancelled');
        }

        public function process_payment($order_id)
        {
            global $woocommerce;

            $order = wc_get_order($order_id);
            $payment_helper = new PaymentHelper($order_id);
            $payment_attempt_response = $payment_helper->initPayment();

            if ((isset($payment_attempt_response['status'])) && ($payment_attempt_response['status'] === 206)) {
                $woocommerce->cart->empty_cart();
                $order->update_status('pending', 'Awaiting payment');
                $order->add_order_note('Awaiting payment');
                WC()->session->set('redirectUrl', $payment_helper->getPaymentUrl($payment_attempt_response));
                return [
                    'result' => 'success',
                    'redirect' => $order->get_checkout_payment_url(true)
                ];
            } elseif (isset($payment_attempt_response['error'])) {
                return $this->process_payment_error('Error: ' . $payment_attempt_response['error'] . '. Order id: ' . $order_id, $order);
            } elseif ($payment_attempt_response['status'] == 204) {
                return $this->process_payment_error("Error: Payment status is " . $payment_attempt_response['status'] . "Order id: $order_id", $order);
            } elseif (is_null($payment_attempt_response)) {
                return $this->process_payment_error("Error: Response from server is empty", $order);
            }
        }

        public function waiting_form($order_id)
        {
            $order = wc_get_order($order_id);
            $redirectUrl = WC()->session->get('redirectUrl', '');
            echo '<script type="text/javascript" src="https://code.jquery.com/jquery-latest.min.js"></script>
                  <script src="https://malsup.github.com/jquery.form.js"></script>
                  <link rel="stylesheet" type="text/css" href=" ' . plugin_dir_url(__FILE__) . 'assets/css/main.css">
                  <script type="text/javascript" src="' . plugin_dir_url(__FILE__) . 'assets/js/main.js"></script>';

            echo '<div>';
            if (isset($redirectUrl['key'])) {
                if ($redirectUrl['key'] == 'offline') {
                    echo '<section><a class="button alt" id="payment_button" href="' . $redirectUrl['src'] . '"> Offline payment</a></section>';
                } elseif ($redirectUrl['key'] == 'qrcode') {
                    echo '<section><a align="center">Scan QR-code with your phone</a><br><img src="' . $redirectUrl['src'] . '"></section>';
                } elseif ($redirectUrl['key'] == 'receiptUrl') {
                    echo '<section><a class="button alt" id="payment_button" href="' . $redirectUrl['src'] . '"> Download invoice </a></section>';
                } elseif ($redirectUrl['key'] == 'redirectUrl') {
                    echo '<section><a class="button alt" id="payment_button" href="' . $redirectUrl['src'] . '"> Pay via ' . $redirectUrl['name'] . ' </a></section>';
                }
            }


            echo '</div>
                  <br>
                  <a class="button alt" id="view-order-button" href="' . $order->get_view_order_url() . '"> View order information </a>
                  <a class="button cancel" id="cancel_button" href="' . $order->get_cancel_order_url() . '"> Cancel and return in cart</a>
                  </br>';
        }

        public function webhook($order_id)
        {
            $event = new EventHandler($order_id);
            $event->handle($_POST);
        }
    }

    require_once 'api/routes.php';
    function apiOrderCheck(WP_REST_Request $request)
    {
        if (!is_user_logged_in()) {
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }

        $params = $request->get_params();
        if (!isset($params['order_id']) && !isset($params['products'])) {
            return new WP_Error('400', 'Invalid parameter request params', ['status' => 400]);
        }

        $products = isset($params['order_id']) ? wc_get_order($params['order_id'])->get_items() : $params['products'] ?? [];
        foreach ($products as $item) {
            $product = wc_get_product($item['product_id']);
            $product_quantity = $product->get_stock_quantity();
            if (!$product->is_in_stock()) {
                return new WP_Error('400', 'The product: ' . $product->get_name() . ' is out of stock');
            }
            if (isset($product_quantity) && $product_quantity < $item['quantity']) {
                return new WP_Error('400', 'The product: ' . $product->get_name() . ' is out of stock. Only ' . $product->get_stock_quantity() . ' left.');
            }
        }

        return ['success' => true];
    }

    function apiOrderPay(WP_REST_Request $request)
    {
        if (!is_user_logged_in()) {
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }

        $params = $request->get_params();

        if (!$params['orderId']) {
            return new WP_Error('400', 'Invalid parameter "orderId"');
        }

        if (!$params['paymentMethodId']) {
            return new WP_Error('400', 'Invalid parameter "paymentMethodId"');
        }

        $provider = new PaymentProvider();

        if (!$order = wc_get_order($params['orderId'])) {
            return new WP_Error('404', 'Order not found');
        }

        $attributes = [];
        $paymentMethods = $provider->getMethodList();
        if (is_array($paymentMethods)) {
            foreach ($paymentMethods as $method) {
                if ($method['id'] === intval($params['paymentMethodId'])) {
                    $paymentMethod = $method['id'];
                    if (count($method['attributes']) != 0) {
                        foreach ($method['attributes'] as $attribute) {
                            if (!empty($params['attributes'])) {
                                $attributes[$attribute['key']] = $params['attributes'][$attribute['key']];
                            }
                        }
                    }
                    break;
                }
            }
        }

        if (!isset($paymentMethod)) {
            return new WP_Error('404', 'Payment method not found');
        }

        $order->add_order_note('Awaiting payment');

        return $provider->initPayment(
            $paymentMethod,
            $order->get_total(),
            $order->get_order_number(),
            $order->get_checkout_order_received_url(),
            $order->get_checkout_order_received_url(),
            $order->get_checkout_order_received_url(),
            $attributes
        );
    }

    //Добавляет возможность возвращать разные размеры картинок в WooCommerce API
    add_filter("woocommerce_rest_prepare_product_object", "prepare_product_images", 10, 3);
    function prepare_product_images($response, $post, $request)
    {
        global $_wp_additional_image_sizes;

        if (empty($response->data)) {
            return $response;
        }

        foreach ($response->data['images'] as $key => $image) {
            $image_urls = [];
            foreach ($_wp_additional_image_sizes as $size => $value) {
                $image_info = wp_get_attachment_image_src($image['id'], $size);
                $response->data['images'][$key][$size] = $image_info[0];
            }
        }
        return $response;
    }

    add_filter('the_title', 'woo_title_order_received', 10, 2);
    function woo_title_order_received($title, $id)
    {
        if (function_exists('is_order_received_page') &&
            is_order_received_page() && get_the_ID() === $id) {
            $title = "Thank you for your order!";
            $current_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
            parse_str($current_link, $params);
            if ($params['order']) {
                wp_redirect(strtok($current_link, '&'));
            }
        }
        return $title;
    }

    add_filter('woocommerce_thankyou_order_received_text', 'woo_change_order_received_text', 10, 2);
    function woo_change_order_received_text($str, $order)
    {
        $status = $order ? $order->get_status() : '';
        switch ($status) {
            case 'success':
            case 'processing':
            case 'completed':
                $message = '&#9989; Your payment has been received successfully!';
                break;
            case 'pending':
                $message = '&#128337; Your payment is processing.';
                break;
            case 'cancelled':
                $message = '&#10060; Your payment is cancelled.';
                break;
            default:
                $message = '';
                break;
        }

        return $message;
    }
}
