<?php

abstract class BaseEvent
{
    protected $provider;

    public function __construct()
    {
        $this->provider = new PaymentProvider();
    }

    public abstract function handle($event);

    /**
     * Check that the order exists in woocommerce
     *
     * @return bool
     */
    protected function isOrderValid($order)
    {
        return ($order ? true : false);
    }

    protected function isProductsInStock($order)
    {
        $order = wc_get_order($order->id);
        $products = $order->get_items();
        foreach ($products as $item) {
            $product = wc_get_product($item['product_id']);
            $product_quantity = $product->get_stock_quantity();
            if (!$product->is_in_stock()) {
                return false;
            }
            if (isset($product_quantity) && $product_quantity < $item['quantity']) {
                return false;
            }
        }
        return true;
    }

    /**
     * Check that the order status is 'Canceled' in woocommerce
     *
     * @return bool
     */
    protected function isOrderCanceled($order)
    {
        return ($order->get_status() === 'canceled');
    }

    /**
     * Check that the order status is 'Accepted' in woocommerce
     *
     * @return bool
     */
    protected function isOrderPaid($order)
    {
        return ($order->get_status() === 'success');
    }

    /**
     * Check that the hash is valid
     *
     * @return bool
     */
    protected function isHashValid($event)
    {
        if (!isset($event['hash'])) {
            return false;
        }

        $hash = $event['hash'];
        unset($event['hash']);
        if ($hash !== $this->provider->hashCallBackRequest($event)) {
            return false;
        }

        return true;
    }

    /**
     * Check that the Woocommerce order amount matсhes amount from Server
     *
     * @return bool
     */
    protected function amountMatching($order, $event)
    {
        return !($order->get_total() != $event['amount']);
    }

    /** Check the Payment status in Server */
    protected function isPaymentStatusSuccess($event)
    {
        $status = $this->provider->getPaymentStatus($event['id']);
        error_log('Invoice Status: ' . $status);
        if ($status != 203 && $status != 205) {
            error_log('Webhooks: The status of order #' . $event['order'] . ' in server is not success.');
            return false;
        }
        return true;
    }
}
