<?php

defined('ABSPATH') or die('No direct script access allowed.');


class EventHandler
{
    private $order_id;
    private $provider;

    public function __construct($order_id)
    {
        $this->order_id = $order_id;
        $this->provider = new PaymentProvider();

        require_once dirname(__FILE__) . '/EventCheck.php';
        require_once dirname(__FILE__) . '/EventPay.php';
    }

    public function handle($event)
    {
        error_log("Webhooks: post request: " . json_encode($event));

        if (!isset($event['method'])) {
            error_log('Webhooks: method not set. ' . 'Request:' . json_encode($event));
            $this->provider->sendResponse(400, 'Webhooks: method not set');
            return false;
        }

        switch ($event['method']) {
            case 'check':
                $message = 'Webhooks: an event CHECK (order#' . $event['order'] . ') has been caught.';
                error_log($message);
                return (new EventCheck())->handle($event);
            case 'pay':
                $message = 'Webhooks: an event PAY (order#' . $event['order'] . ') has been caught.';
                error_log($message);
                return (new EventPay())->handle($event);
            case 'payment.rejected':
                $message = 'Webhooks: an event REJECT (order#' . $event['order'] . ') has been caught.';
                error_log($message);
                $order = wc_get_order($event['order']);
                $order->update_status('cancelled', $message);
                $order->add_order_note('Cancelled');
                $this->provider->sendResponse(270, 'Notification received');
                return true;
            default:
                error_log('Webhooks: unknown method: ' . $event['method']);
                $this->provider->sendResponse(400, 'Webhooks: unknown method: ' . $event['method']);
                return false;
        }
    }
}
